"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executableOpenAPIMiddlewareRequestBody = void 0;
const openapi_document_local_ref_resolver_1 = require("openapi-document-local-ref-resolver");
const executable_openapi_parameters_1 = require("executable-openapi-parameters");
const _2020_1 = require("ajv/dist/2020");
const memoize = require("memoizee");
const mimeMatch = require("mime-match");
const ts_invariant_1 = require("ts-invariant");
const errors_1 = require("./errors");
const executableOpenAPIMiddlewareRequestBody = ({ documentRefResolver, formatErrorResponse = errors_1.formatInvalidRequestBodyResponse, formats } = {}) => {
    const ajv = new _2020_1.default({
        coerceTypes: true,
        useDefaults: true,
        removeAdditional: true,
        formats
    });
    const compileAjvBasedValidateCached = memoize(executable_openapi_parameters_1.compileAjvBasedValidate);
    return (handle, parameters, body, context, info) => __awaiter(void 0, void 0, void 0, function* () {
        const resolveRef = documentRefResolver !== null && documentRefResolver !== void 0 ? documentRefResolver : (0, openapi_document_local_ref_resolver_1.localRefResolver)(info.document);
        const maybeRequestBody = info.operationObject.requestBody;
        const requestBody = maybeRequestBody !== undefined && '$ref' in maybeRequestBody ? yield resolveRef(maybeRequestBody.$ref) : maybeRequestBody;
        if (requestBody === undefined || Object.keys(requestBody.content).length === 0) {
            // no requestBody, we discard the body
            return yield handle(parameters, undefined, context, info);
        }
        if (requestBody.required === true && body === undefined) {
            const error = new errors_1.InvalidRequestBodyError('is missing');
            return yield formatErrorResponse(error, info.operationObject.responses);
        }
        if (body === undefined) {
            return yield handle(parameters, undefined, context, info);
        }
        // if body is present, executionRequest.body should be there
        (0, ts_invariant_1.default)(info.executionRequest.body);
        const bodyMediaType = info.executionRequest.body.mediaType;
        const mediaTypes = Object.keys(requestBody.content);
        const matchedMediaTypes = mediaTypes.filter((mediaType) => mimeMatch(bodyMediaType, mediaType));
        if (matchedMediaTypes.length === 0) {
            const error = new errors_1.InvalidRequestBodyError(`Media type ${bodyMediaType} is not acceptable`);
            return yield formatErrorResponse(error, info.operationObject.responses);
        }
        // todo: https://git.io/JyDl0 for requests that match multiple keys, only the most specific key is applicable. e.g. text/plain overrides text/*
        const contentSpec = requestBody.content[matchedMediaTypes[0]];
        if (contentSpec.schema !== undefined) {
            const validate = compileAjvBasedValidateCached(ajv, contentSpec.schema);
            const { pass, errors, data: newBody } = validate(body);
            if (!pass) {
                (0, ts_invariant_1.default)(errors, '`validate` should return defined `errors` when `pass` is true');
                const error = new errors_1.InvalidRequestBodyError(errors);
                return yield formatErrorResponse(error, info.operationObject.responses);
            }
            return yield handle(parameters, newBody, context, info);
        }
        return yield handle(parameters, body, context, info);
    });
};
exports.executableOpenAPIMiddlewareRequestBody = executableOpenAPIMiddlewareRequestBody;
//# sourceMappingURL=middleware.js.map